@@ -1,9 +1,9 @@
 /* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -22,6 +21,7 @@
 #include <string.h>
 #include <errno.h>
 
+#include <cairo.h>
 #include <glib-object.h>
 #include <glib/gstdio.h>
 
@@ -39,7 +39,8 @@
 
 
 GList *
-gimp_gradient_load (const gchar  *filename,
+gimp_gradient_load (GimpContext  *context,
+                    const gchar  *filename,
                     GError      **error)
 {
   GimpGradient        *gradient;
@@ -311,7 +312,8 @@ static const GMarkupParser markup_parser =
 
 
 GList *
-gimp_gradient_load_svg (const gchar  *filename,
+gimp_gradient_load_svg (GimpContext  *context,
+                        const gchar  *filename,
                         GError      **error)
 {
   GimpXmlParser *xml_parser;
@@ -520,8 +522,12 @@ svg_parse_gradient_stop_style (SvgStop     *stop,
 
       if (end > sep && sep > style)
         {
-          gchar *name  = g_strndup (style, sep - style);
-          gchar *value = g_strndup (++sep, end - sep - (*end == ';' ? 1 : 0));
+          gchar *name;
+          gchar *value;
+
+          name = g_strndup (style, sep - style);
+          sep++;
+          value = g_strndup (sep, end - sep - (*end == ';' ? 1 : 0));
 
           svg_parse_gradient_stop_style_prop (stop, name, value);
 
