@@ -3,9 +3,9 @@
  *
  * gimpplugin-message.c
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,15 +14,14 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
 #include <string.h>
 
-#include <glib-object.h>
+#include <gegl.h>
 
 #include "libgimpbase/gimpbase.h"
 #include "libgimpbase/gimpprotocol.h"
@@ -96,7 +95,7 @@ gimp_plug_in_handle_message (GimpPlugIn      *plug_in,
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
                     "Plug-In \"%s\"\n(%s)\n\n"
                     "sent a CONFIG message.  This should not happen.",
-                    gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                    gimp_object_get_name (plug_in),
                     gimp_filename_to_utf8 (plug_in->prog));
       gimp_plug_in_close (plug_in, TRUE);
       break;
@@ -109,7 +108,7 @@ gimp_plug_in_handle_message (GimpPlugIn      *plug_in,
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
                     "Plug-In \"%s\"\n(%s)\n\n"
                     "sent a TILE_ACK message.  This should not happen.",
-                    gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                    gimp_object_get_name (plug_in),
                     gimp_filename_to_utf8 (plug_in->prog));
       gimp_plug_in_close (plug_in, TRUE);
       break;
@@ -118,7 +117,7 @@ gimp_plug_in_handle_message (GimpPlugIn      *plug_in,
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
                     "Plug-In \"%s\"\n(%s)\n\n"
                     "sent a TILE_DATA message.  This should not happen.",
-                    gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                    gimp_object_get_name (plug_in),
                     gimp_filename_to_utf8 (plug_in->prog));
       gimp_plug_in_close (plug_in, TRUE);
       break;
@@ -135,7 +134,7 @@ gimp_plug_in_handle_message (GimpPlugIn      *plug_in,
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
                     "Plug-In \"%s\"\n(%s)\n\n"
                     "sent a TEMP_PROC_RUN message.  This should not happen.",
-                    gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                    gimp_object_get_name (plug_in),
                     gimp_filename_to_utf8 (plug_in->prog));
       gimp_plug_in_close (plug_in, TRUE);
       break;
@@ -237,7 +236,7 @@ gimp_plug_in_handle_tile_put (GimpPlugIn *plug_in,
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
                     "Plug-In \"%s\"\n(%s)\n\n"
                     "tried writing to invalid drawable %d (killing)",
-                    gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                    gimp_object_get_name (plug_in),
                     gimp_filename_to_utf8 (plug_in->prog),
                     tile_info->drawable_ID);
       gimp_plug_in_close (plug_in, TRUE);
@@ -249,7 +248,7 @@ gimp_plug_in_handle_tile_put (GimpPlugIn *plug_in,
                     "Plug-In \"%s\"\n(%s)\n\n"
                     "tried writing to drawable %d which was removed "
                     "from the image (killing)",
-                    gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                    gimp_object_get_name (plug_in),
                     gimp_filename_to_utf8 (plug_in->prog),
                     tile_info->drawable_ID);
       gimp_plug_in_close (plug_in, TRUE);
@@ -258,12 +257,40 @@ gimp_plug_in_handle_tile_put (GimpPlugIn *plug_in,
 
   if (tile_info->shadow)
     {
+      /*  don't check whether the drawable is a group or locked here,
+       *  the plugin will get a proper error message when it tries to
+       *  merge the shadow tiles, which is much better than just
+       *  killing it.
+       */
       tm = gimp_drawable_get_shadow_tiles (drawable);
 
       gimp_plug_in_cleanup_add_shadow (plug_in, drawable);
     }
   else
     {
+      if (gimp_item_is_content_locked (GIMP_ITEM (drawable)))
+        {
+          gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
+                        "Plug-In \"%s\"\n(%s)\n\n"
+                        "tried writing to a locked drawable %d (killing)",
+                        gimp_object_get_name (plug_in),
+                        gimp_filename_to_utf8 (plug_in->prog),
+                        tile_info->drawable_ID);
+          gimp_plug_in_close (plug_in, TRUE);
+          return;
+        }
+      else if (gimp_viewable_get_children (GIMP_VIEWABLE (drawable)))
+        {
+          gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
+                        "Plug-In \"%s\"\n(%s)\n\n"
+                        "tried writing to a group layer %d (killing)",
+                        gimp_object_get_name (plug_in),
+                        gimp_filename_to_utf8 (plug_in->prog),
+                        tile_info->drawable_ID);
+          gimp_plug_in_close (plug_in, TRUE);
+          return;
+        }
+
       tm = gimp_drawable_get_tiles (drawable);
     }
 
@@ -274,7 +301,7 @@ gimp_plug_in_handle_tile_put (GimpPlugIn *plug_in,
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
                     "Plug-In \"%s\"\n(%s)\n\n"
                     "requested invalid tile (killing)",
-                    gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                    gimp_object_get_name (plug_in),
                     gimp_filename_to_utf8 (plug_in->prog));
       gimp_plug_in_close (plug_in, TRUE);
       return;
@@ -319,7 +346,7 @@ gimp_plug_in_handle_tile_get (GimpPlugIn *plug_in,
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
                     "Plug-In \"%s\"\n(%s)\n\n"
                     "tried reading from invalid drawable %d (killing)",
-                    gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                    gimp_object_get_name (plug_in),
                     gimp_filename_to_utf8 (plug_in->prog),
                     request->drawable_ID);
       gimp_plug_in_close (plug_in, TRUE);
@@ -331,7 +358,7 @@ gimp_plug_in_handle_tile_get (GimpPlugIn *plug_in,
                     "Plug-In \"%s\"\n(%s)\n\n"
                     "tried reading from drawable %d which was removed "
                     "from the image (killing)",
-                    gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                    gimp_object_get_name (plug_in),
                     gimp_filename_to_utf8 (plug_in->prog),
                     request->drawable_ID);
       gimp_plug_in_close (plug_in, TRUE);
@@ -356,7 +383,7 @@ gimp_plug_in_handle_tile_get (GimpPlugIn *plug_in,
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
                     "Plug-In \"%s\"\n(%s)\n\n"
                     "requested invalid tile (killing)",
-                    gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                    gimp_object_get_name (plug_in),
                     gimp_filename_to_utf8 (plug_in->prog));
       gimp_plug_in_close (plug_in, TRUE);
       return;
@@ -479,7 +506,7 @@ gimp_plug_in_handle_proc_run (GimpPlugIn *plug_in,
                             "Plug-In \"%s\"\n(%s)\n"
                             "called deprecated procedure '%s'.\n"
                             "It should call '%s' instead!",
-                            gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                            gimp_object_get_name (plug_in),
                             gimp_filename_to_utf8 (plug_in->prog),
                             canonical, proc_name);
             }
@@ -494,7 +521,7 @@ gimp_plug_in_handle_proc_run (GimpPlugIn *plug_in,
               gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_WARNING,
                             "Plug-In \"%s\"\n(%s)\n"
                             "called deprecated procedure '%s'.",
-                            gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                            gimp_object_get_name (plug_in),
                             gimp_filename_to_utf8 (plug_in->prog),
                             canonical);
             }
@@ -504,7 +531,7 @@ gimp_plug_in_handle_proc_run (GimpPlugIn *plug_in,
                             "WARNING: Plug-In \"%s\"\n(%s)\n"
                             "called deprecated procedure '%s'.\n"
                             "It should call '%s' instead!",
-                            gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                            gimp_object_get_name (plug_in),
                             gimp_filename_to_utf8 (plug_in->prog),
                             canonical, procedure->deprecated);
             }
@@ -581,17 +608,27 @@ gimp_plug_in_handle_proc_return (GimpPlugIn   *plug_in,
 
   g_return_if_fail (proc_return != NULL);
 
+  proc_frame->return_vals =
+    plug_in_params_to_args (proc_frame->procedure->values,
+                            proc_frame->procedure->num_values,
+                            proc_return->params,
+                            proc_return->nparams,
+                            TRUE, TRUE);
+
   if (proc_frame->main_loop)
     {
-      proc_frame->return_vals =
-        plug_in_params_to_args (proc_frame->procedure->values,
-                                proc_frame->procedure->num_values,
-                                proc_return->params,
-                                proc_return->nparams,
-                                TRUE, TRUE);
-
       g_main_loop_quit (proc_frame->main_loop);
     }
+  else
+    {
+      /*  the plug-in is run asynchronously, so display its error
+       *  messages here because nobody else will do it
+       */
+      gimp_plug_in_procedure_handle_return_values (GIMP_PLUG_IN_PROCEDURE (proc_frame->procedure),
+                                                   plug_in->manager->gimp,
+                                                   proc_frame->progress,
+                                                   proc_frame->return_vals);
+    }
 
   gimp_plug_in_close (plug_in, FALSE);
 }
@@ -622,7 +659,7 @@ gimp_plug_in_handle_temp_proc_return (GimpPlugIn   *plug_in,
                     "Plug-In \"%s\"\n(%s)\n\n"
                     "sent a TEMP_PROC_RETURN message while not running "
                     "a temporary procedure.  This should not happen.",
-                    gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                    gimp_object_get_name (plug_in),
                     gimp_filename_to_utf8 (plug_in->prog));
       gimp_plug_in_close (plug_in, TRUE);
     }
@@ -661,7 +698,7 @@ gimp_plug_in_handle_proc_install (GimpPlugIn    *plug_in,
                         "attempted to install procedure \"%s\" "
                         "which fails to comply with the array parameter "
                         "passing standard.  Argument %d is noncompliant.",
-                        gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                        gimp_object_get_name (plug_in),
                         gimp_filename_to_utf8 (plug_in->prog),
                         canonical, i);
           g_free (canonical);
@@ -720,7 +757,7 @@ gimp_plug_in_handle_proc_install (GimpPlugIn    *plug_in,
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
                     "Plug-In \"%s\"\n(%s)\n\n"
                     "attempted to install a procedure NULL parameter name.",
-                    gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                    gimp_object_get_name (plug_in),
                     gimp_filename_to_utf8 (plug_in->prog));
       g_free (canonical);
       return;
@@ -731,7 +768,7 @@ gimp_plug_in_handle_proc_install (GimpPlugIn    *plug_in,
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
                     "Plug-In \"%s\"\n(%s)\n\n"
                     "attempted to install a procedure with invalid UTF-8 strings.",
-                    gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                    gimp_object_get_name (plug_in),
                     gimp_filename_to_utf8 (plug_in->prog));
       g_free (canonical);
       return;
@@ -803,8 +840,9 @@ gimp_plug_in_handle_proc_install (GimpPlugIn    *plug_in,
                                                       proc_install->menu_path,
                                                       &error))
             {
-              gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_WARNING,
-                            "%s", error->message);
+              gimp_message_literal (plug_in->manager->gimp,
+				    NULL, GIMP_MESSAGE_WARNING,
+				    error->message);
               g_clear_error (&error);
             }
         }
@@ -864,7 +902,7 @@ gimp_plug_in_handle_extension_ack (GimpPlugIn *plug_in)
                     "Plug-In \"%s\"\n(%s)\n\n"
                     "sent an EXTENSION_ACK message while not being started "
                     "as an extension.  This should not happen.",
-                    gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                    gimp_object_get_name (plug_in),
                     gimp_filename_to_utf8 (plug_in->prog));
       gimp_plug_in_close (plug_in, TRUE);
     }
@@ -883,7 +921,7 @@ gimp_plug_in_handle_has_init (GimpPlugIn *plug_in)
                     "Plug-In \"%s\"\n(%s)\n\n"
                     "sent an HAS_INIT message while not in query().  "
                     "This should not happen.",
-                    gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                    gimp_object_get_name (plug_in),
                     gimp_filename_to_utf8 (plug_in->prog));
       gimp_plug_in_close (plug_in, TRUE);
     }
