@@ -6,9 +6,9 @@
  * Copyright (C) 1999 Thom van Os <thom@vanos.com>
  * Copyright (C) 2006 Loren Merritt
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -17,8 +17,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * To do:
  *      - support for horizontal or vertical only blur
@@ -39,6 +38,7 @@
 
 #define PLUG_IN_PROC   "plug-in-sel-gauss"
 #define PLUG_IN_BINARY "blur-gauss-selective"
+#define PLUG_IN_ROLE   "gimp-blur-gauss-selective"
 
 #ifndef ALWAYS_INLINE
 #if defined(__GNUC__) && (__GNUC__ > 3 || __GNUC__ == 3 && __GNUC_MINOR__ > 0)
@@ -93,7 +93,7 @@ query (void)
 {
   static const GimpParamDef args[] =
   {
-    { GIMP_PDB_INT32,    "run-mode",  "Interactive, non-interactive" },
+    { GIMP_PDB_INT32,    "run-mode",  "The run mode { RUN-INTERACTIVE (0), RUN-NONINTERACTIVE (1) }" },
     { GIMP_PDB_IMAGE,    "image",     "Input image (unused)"         },
     { GIMP_PDB_DRAWABLE, "drawable",  "Input drawable"               },
     { GIMP_PDB_FLOAT,    "radius",    "Radius of gaussian blur (in pixels, > 0.0)" },
@@ -229,7 +229,7 @@ sel_gauss_dialog (GimpDrawable *drawable)
 
   gimp_ui_init (PLUG_IN_BINARY, FALSE);
 
-  dialog = gimp_dialog_new (_("Selective Gaussian Blur"), PLUG_IN_BINARY,
+  dialog = gimp_dialog_new (_("Selective Gaussian Blur"), PLUG_IN_ROLE,
                             NULL, 0,
                             gimp_standard_help_func, PLUG_IN_PROC,
 
@@ -245,9 +245,10 @@ sel_gauss_dialog (GimpDrawable *drawable)
 
   gimp_window_set_transient (GTK_WINDOW (dialog));
 
-  main_vbox = gtk_vbox_new (FALSE, 12);
+  main_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 12);
   gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 12);
-  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), main_vbox);
+  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
+                      main_vbox, TRUE, TRUE, 0);
   gtk_widget_show (main_vbox);
 
   preview = gimp_drawable_preview_new (drawable, NULL);
@@ -731,19 +732,19 @@ sel_gauss (GimpDrawable *drawable,
            gint          maxdelta)
 {
   GimpPixelRgn src_rgn, dest_rgn;
-  gint         width, height;
   gint         bytes;
   gboolean     has_alpha;
   guchar      *dest;
   guchar      *src;
-  gint         x1, y1, x2, y2;
+  gint         x, y;
+  gint         width, height;
   gdouble     *mat;
   gint         numrad;
 
-  gimp_drawable_mask_bounds (drawable->drawable_id, &x1, &y1, &x2, &y2);
+  if (! gimp_drawable_mask_intersect (drawable->drawable_id,
+                                      &x, &y, &width, &height))
+    return;
 
-  width     = x2 - x1;
-  height    = y2 - y1;
   bytes     = drawable->bpp;
   has_alpha = gimp_drawable_has_alpha (drawable->drawable_id);
 
@@ -757,20 +758,21 @@ sel_gauss (GimpDrawable *drawable,
   dest = g_new (guchar, width * height * bytes);
 
   gimp_pixel_rgn_init (&src_rgn,
-                       drawable, x1, y1, width, height, FALSE, FALSE);
-  gimp_pixel_rgn_get_rect (&src_rgn, src, x1, y1, width, height);
+                       drawable, x, y, width, height, FALSE, FALSE);
+  gimp_pixel_rgn_get_rect (&src_rgn, src, x, y, width, height);
 
   matrixmult (src, dest, width, height, mat, numrad,
               bytes, has_alpha, maxdelta, FALSE);
+  gimp_progress_update (1.0);
 
   gimp_pixel_rgn_init (&dest_rgn,
-                       drawable, x1, y1, width, height, TRUE, TRUE);
-  gimp_pixel_rgn_set_rect (&dest_rgn, dest, x1, y1, width, height);
+                       drawable, x, y, width, height, TRUE, TRUE);
+  gimp_pixel_rgn_set_rect (&dest_rgn, dest, x, y, width, height);
 
   /*  merge the shadow, update the drawable  */
   gimp_drawable_flush (drawable);
   gimp_drawable_merge_shadow (drawable->drawable_id, TRUE);
-  gimp_drawable_update (drawable->drawable_id, x1, y1, width, height);
+  gimp_drawable_update (drawable->drawable_id, x, y, width, height);
 
   /* free up buffers */
   g_free (src);
@@ -783,7 +785,7 @@ preview_update (GimpPreview *preview)
 {
   GimpDrawable  *drawable;
   glong          bytes;
-  gint           x1, y1;
+  gint           x, y;
   guchar        *render_buffer;  /* Buffer to hold rendered image */
   gint           width;          /* Width of preview widget */
   gint           height;         /* Height of preview widget */
@@ -803,12 +805,12 @@ preview_update (GimpPreview *preview)
   /*
    * Setup for filter...
    */
-  gimp_preview_get_position (preview, &x1, &y1);
+  gimp_preview_get_position (preview, &x, &y);
   gimp_preview_get_size (preview, &width, &height);
 
   /* initialize pixel regions */
   gimp_pixel_rgn_init (&srcPR, drawable,
-                       x1, y1, width, height,
+                       x, y, width, height,
                        FALSE, FALSE);
   render_buffer = g_new (guchar, width * height * bytes);
 
@@ -817,7 +819,7 @@ preview_update (GimpPreview *preview)
   src = g_new (guchar, width * height * bytes + 16);
 
   /* render image */
-  gimp_pixel_rgn_get_rect (&srcPR, src, x1, y1, width, height);
+  gimp_pixel_rgn_get_rect (&srcPR, src, x, y, width, height);
   has_alpha = gimp_drawable_has_alpha (drawable->drawable_id);
 
   radius = fabs (bvals.radius) + 1.0;
