@@ -1,24 +1,22 @@
-/* A Bison parser, made by GNU Bison 2.3.  */
+/* A Bison parser, made by GNU Bison 2.4.3.  */
 
 /* Skeleton implementation for Bison's Yacc-like parsers in C
-
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
-   Free Software Foundation, Inc.
-
-   This program is free software; you can redistribute it and/or modify
+   
+      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
+   2009, 2010 Free Software Foundation, Inc.
+   
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
-
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+   
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-
+   
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* As a special exception, you may create a larger work that contains
    part or all of the Bison parser skeleton and distribute that work
@@ -29,7 +27,7 @@
    special exception, which will cause the skeleton and the resulting
    Bison output files to be licensed under the GNU General Public
    License without this special exception.
-
+   
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
@@ -47,7 +45,7 @@
 #define YYBISON 1
 
 /* Bison version.  */
-#define YYBISON_VERSION "2.3"
+#define YYBISON_VERSION "2.4.3"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -55,17 +53,96 @@
 /* Pure parsers.  */
 #define YYPURE 0
 
+/* Push parsers.  */
+#define YYPUSH 0
+
+/* Pull parsers.  */
+#define YYPULL 1
+
 /* Using locations.  */
 #define YYLSP_NEEDED 0
 
 /* Substitute the variable and function names.  */
-#define yyparse csim_parse
-#define yylex   csim_lex
-#define yyerror csim_error
-#define yylval  csim_lval
-#define yychar  csim_char
-#define yydebug csim_debug
-#define yynerrs csim_nerrs
+#define yyparse         csim_parse
+#define yylex           csim_lex
+#define yyerror         csim_error
+#define yylval          csim_lval
+#define yychar          csim_char
+#define yydebug         csim_debug
+#define yynerrs         csim_nerrs
+
+
+/* Copy the first part of user declarations.  */
+
+/* Line 189 of yacc.c  */
+#line 1 "imap_csim.y"
+
+/*
+ * This is a plug-in for GIMP.
+ *
+ * Generates clickable image maps.
+ *
+ * Copyright (C) 1998-2005 Maurits Rijk  lpeek.mrijk@consunet.nl
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ */
+
+#include <stdlib.h>
+#include <string.h>
+
+#include <glib/gstdio.h>
+
+#include <gtk/gtk.h>
+
+#include "imap_circle.h"
+#include "imap_file.h"
+#include "imap_main.h"
+#include "imap_polygon.h"
+#include "imap_rectangle.h"
+#include "imap_string.h"
+
+extern int csim_lex(void);
+extern int csim_restart(FILE *csim_in);
+static void csim_error(char* s);
+
+static enum {UNDEFINED, RECTANGLE, CIRCLE, POLYGON} current_type;
+static Object_t *current_object;
+static MapInfo_t *_map_info;
+
+
+
+/* Line 189 of yacc.c  */
+#line 128 "y.tab.c"
+
+/* Enabling traces.  */
+#ifndef YYDEBUG
+# define YYDEBUG 0
+#endif
+
+/* Enabling verbose error messages.  */
+#ifdef YYERROR_VERBOSE
+# undef YYERROR_VERBOSE
+# define YYERROR_VERBOSE 1
+#else
+# define YYERROR_VERBOSE 0
+#endif
+
+/* Enabling the token table.  */
+#ifndef YYTOKEN_TABLE
+# define YYTOKEN_TABLE 0
+#endif
 
 
 /* Tokens.  */
@@ -133,97 +210,33 @@
 
 
 
-/* Copy the first part of user declarations.  */
-#line 1 "imap_csim.y"
-
-/*
- * This is a plug-in for GIMP.
- *
- * Generates clickable image maps.
- *
- * Copyright (C) 1998-2005 Maurits Rijk  lpeek.mrijk@consunet.nl
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- */
-
-#include <stdlib.h>
-#include <string.h>
-
-#include <glib/gstdio.h>
-
-#include <gtk/gtk.h>
-
-#include "imap_circle.h"
-#include "imap_file.h"
-#include "imap_main.h"
-#include "imap_polygon.h"
-#include "imap_rectangle.h"
-#include "imap_string.h"
-
-extern int csim_lex();
-extern int csim_restart();
-static void csim_error(char* s);
-
-static enum {UNDEFINED, RECTANGLE, CIRCLE, POLYGON} current_type;
-static Object_t *current_object;
-static MapInfo_t *_map_info;
-
-
-
-/* Enabling traces.  */
-#ifndef YYDEBUG
-# define YYDEBUG 0
-#endif
-
-/* Enabling verbose error messages.  */
-#ifdef YYERROR_VERBOSE
-# undef YYERROR_VERBOSE
-# define YYERROR_VERBOSE 1
-#else
-# define YYERROR_VERBOSE 0
-#endif
-
-/* Enabling the token table.  */
-#ifndef YYTOKEN_TABLE
-# define YYTOKEN_TABLE 0
-#endif
-
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 49 "imap_csim.y"
 {
+
+/* Line 214 of yacc.c  */
+#line 48 "imap_csim.y"
+
   int val;
   double value;
   char *id;
-}
-/* Line 187 of yacc.c.  */
-#line 214 "y.tab.c"
-	YYSTYPE;
+
+
+
+/* Line 214 of yacc.c  */
+#line 228 "y.tab.c"
+} YYSTYPE;
+# define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
-# define YYSTYPE_IS_TRIVIAL 1
 #endif
 
 
-
 /* Copy the second part of user declarations.  */
 
 
-/* Line 216 of yacc.c.  */
-#line 227 "y.tab.c"
+/* Line 264 of yacc.c  */
+#line 240 "y.tab.c"
 
 #ifdef short
 # undef short
@@ -273,7 +286,7 @@ typedef short int yytype_int16;
 #define YYSIZE_MAXIMUM ((YYSIZE_T) -1)
 
 #ifndef YY_
-# if YYENABLE_NLS
+# if defined YYENABLE_NLS && YYENABLE_NLS
 #  if ENABLE_NLS
 #   include <libintl.h> /* INFRINGES ON USER NAME SPACE */
 #   define YY_(msgid) dgettext ("bison-runtime", msgid)
@@ -298,14 +311,14 @@ typedef short int yytype_int16;
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 static int
-YYID (int i)
+YYID (int yyi)
 #else
 static int
-YYID (i)
-    int i;
+YYID (yyi)
+    int yyi;
 #endif
 {
-  return i;
+  return yyi;
 }
 #endif
 
@@ -386,9 +399,9 @@ void free (void *); /* INFRINGES ON USER NAME SPACE */
 /* A type that is properly aligned for any stack member.  */
 union yyalloc
 {
-  yytype_int16 yyss;
-  YYSTYPE yyvs;
-  };
+  yytype_int16 yyss_alloc;
+  YYSTYPE yyvs_alloc;
+};
 
 /* The size of the maximum gap between one aligned stack and the next.  */
 # define YYSTACK_GAP_MAXIMUM (sizeof (union yyalloc) - 1)
@@ -422,12 +435,12 @@ union yyalloc
    elements in the stack, and YYPTR gives the new location of the
    stack.  Advance YYPTR to a properly aligned location for the next
    stack.  */
-# define YYSTACK_RELOCATE(Stack)					\
+# define YYSTACK_RELOCATE(Stack_alloc, Stack)				\
     do									\
       {									\
 	YYSIZE_T yynewbytes;						\
-	YYCOPY (&yyptr->Stack, Stack, yysize);				\
-	Stack = &yyptr->Stack;						\
+	YYCOPY (&yyptr->Stack_alloc, Stack, yysize);			\
+	Stack = &yyptr->Stack_alloc;					\
 	yynewbytes = yystacksize * sizeof (*Stack) + YYSTACK_GAP_MAXIMUM; \
 	yyptr += yynewbytes / sizeof (*yyptr);				\
       }									\
@@ -528,12 +541,12 @@ static const yytype_int8 yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint16 yyrline[] =
 {
-       0,    66,    66,    69,    76,    77,    80,    81,    82,    83,
-      84,    87,    93,    99,   103,   110,   117,   118,   121,   122,
-     123,   126,   132,   139,   150,   151,   154,   161,   162,   165,
-     166,   169,   170,   171,   172,   173,   174,   175,   176,   177,
-     178,   181,   199,   259,   270,   275,   276,   282,   289,   296,
-     303,   310,   317,   324
+       0,    65,    65,    68,    75,    76,    79,    80,    81,    82,
+      83,    86,    92,    98,   102,   109,   116,   117,   120,   121,
+     122,   125,   131,   138,   149,   150,   153,   160,   161,   164,
+     165,   168,   169,   170,   171,   172,   173,   174,   175,   176,
+     177,   180,   198,   258,   269,   274,   275,   281,   288,   295,
+     302,   309,   316,   323
 };
 #endif
 
@@ -705,9 +718,18 @@ static const yytype_uint8 yystos[] =
 
 /* Like YYERROR except do call yyerror.  This remains here temporarily
    to ease the transition to the new meaning of YYERROR, for GCC.
-   Once GCC version 2 has supplanted version 1, this can go.  */
+   Once GCC version 2 has supplanted version 1, this can go.  However,
+   YYFAIL appears to be in use.  Nevertheless, it is formally deprecated
+   in Bison 2.4.2's NEWS entry, where a plan to phase it out is
+   discussed.  */
 
 #define YYFAIL		goto yyerrlab
+#if defined YYFAIL
+  /* This is here to suppress warnings from the GCC cpp's
+     -Wunused-macros.  Normally we don't worry about that warning, but
+     some users do, and we want to make it easy for users to remove
+     YYFAIL uses, which will produce warnings from Bison 2.5.  */
+#endif
 
 #define YYRECOVERING()  (!!yyerrstatus)
 
@@ -764,7 +786,7 @@ while (YYID (0))
    we won't break user code: when these are the locations we know.  */
 
 #ifndef YY_LOCATION_PRINT
-# if YYLTYPE_IS_TRIVIAL
+# if defined YYLTYPE_IS_TRIVIAL && YYLTYPE_IS_TRIVIAL
 #  define YY_LOCATION_PRINT(File, Loc)			\
      fprintf (File, "%d.%d-%d.%d",			\
 	      (Loc).first_line, (Loc).first_column,	\
@@ -875,17 +897,20 @@ yy_symbol_print (yyoutput, yytype, yyvaluep)
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 static void
-yy_stack_print (yytype_int16 *bottom, yytype_int16 *top)
+yy_stack_print (yytype_int16 *yybottom, yytype_int16 *yytop)
 #else
 static void
-yy_stack_print (bottom, top)
-    yytype_int16 *bottom;
-    yytype_int16 *top;
+yy_stack_print (yybottom, yytop)
+    yytype_int16 *yybottom;
+    yytype_int16 *yytop;
 #endif
 {
   YYFPRINTF (stderr, "Stack now");
-  for (; bottom <= top; ++bottom)
-    YYFPRINTF (stderr, " %d", *bottom);
+  for (; yybottom <= yytop; yybottom++)
+    {
+      int yybot = *yybottom;
+      YYFPRINTF (stderr, " %d", yybot);
+    }
   YYFPRINTF (stderr, "\n");
 }
 
@@ -919,11 +944,11 @@ yy_reduce_print (yyvsp, yyrule)
   /* The symbols being reduced.  */
   for (yyi = 0; yyi < yynrhs; yyi++)
     {
-      fprintf (stderr, "   $%d = ", yyi + 1);
+      YYFPRINTF (stderr, "   $%d = ", yyi + 1);
       yy_symbol_print (stderr, yyrhs[yyprhs[yyrule] + yyi],
 		       &(yyvsp[(yyi + 1) - (yynrhs)])
 		       		       );
-      fprintf (stderr, "\n");
+      YYFPRINTF (stderr, "\n");
     }
 }
 
@@ -1203,10 +1228,8 @@ yydestruct (yymsg, yytype, yyvaluep)
 	break;
     }
 }
-
 
 /* Prevent warnings from -Wmissing-prototypes.  */
-
 #ifdef YYPARSE_PARAM
 #if defined __STDC__ || defined __cplusplus
 int yyparse (void *YYPARSE_PARAM);
@@ -1222,11 +1245,10 @@ int yyparse ();
 #endif /* ! YYPARSE_PARAM */
 
 
-
-/* The look-ahead symbol.  */
+/* The lookahead symbol.  */
 int yychar;
 
-/* The semantic value of the look-ahead symbol.  */
+/* The semantic value of the lookahead symbol.  */
 YYSTYPE yylval;
 
 /* Number of syntax errors so far.  */
@@ -1234,9 +1256,9 @@ int yynerrs;
 
 
 
-/*----------.
-| yyparse.  |
-`----------*/
+/*-------------------------.
+| yyparse or yypush_parse.  |
+`-------------------------*/
 
 #ifdef YYPARSE_PARAM
 #if (defined __STDC__ || defined __C99__FUNC__ \
@@ -1261,13 +1283,38 @@ yyparse ()
 #endif
 {
 
-  int yystate;
+
+    int yystate;
+    /* Number of tokens to shift before error messages enabled.  */
+    int yyerrstatus;
+
+    /* The stacks and their tools:
+       `yyss': related to states.
+       `yyvs': related to semantic values.
+
+       Refer to the stacks thru separate pointers, to allow yyoverflow
+       to reallocate them elsewhere.  */
+
+    /* The state stack.  */
+    yytype_int16 yyssa[YYINITDEPTH];
+    yytype_int16 *yyss;
+    yytype_int16 *yyssp;
+
+    /* The semantic value stack.  */
+    YYSTYPE yyvsa[YYINITDEPTH];
+    YYSTYPE *yyvs;
+    YYSTYPE *yyvsp;
+
+    YYSIZE_T yystacksize;
+
   int yyn;
   int yyresult;
-  /* Number of tokens to shift before error messages enabled.  */
-  int yyerrstatus;
-  /* Look-ahead token as an internal (translated) token number.  */
-  int yytoken = 0;
+  /* Lookahead token as an internal (translated) token number.  */
+  int yytoken;
+  /* The variables used to return semantic value and location from the
+     action routines.  */
+  YYSTYPE yyval;
+
 #if YYERROR_VERBOSE
   /* Buffer for error messages, and its allocated size.  */
   char yymsgbuf[128];
@@ -1275,51 +1322,28 @@ yyparse ()
   YYSIZE_T yymsg_alloc = sizeof yymsgbuf;
 #endif
 
-  /* Three stacks and their tools:
-     `yyss': related to states,
-     `yyvs': related to semantic values,
-     `yyls': related to locations.
-
-     Refer to the stacks thru separate pointers, to allow yyoverflow
-     to reallocate them elsewhere.  */
-
-  /* The state stack.  */
-  yytype_int16 yyssa[YYINITDEPTH];
-  yytype_int16 *yyss = yyssa;
-  yytype_int16 *yyssp;
-
-  /* The semantic value stack.  */
-  YYSTYPE yyvsa[YYINITDEPTH];
-  YYSTYPE *yyvs = yyvsa;
-  YYSTYPE *yyvsp;
-
-
-
 #define YYPOPSTACK(N)   (yyvsp -= (N), yyssp -= (N))
 
-  YYSIZE_T yystacksize = YYINITDEPTH;
-
-  /* The variables used to return semantic value and location from the
-     action routines.  */
-  YYSTYPE yyval;
-
-
   /* The number of symbols on the RHS of the reduced rule.
      Keep to zero when no symbol should be popped.  */
   int yylen = 0;
 
+  yytoken = 0;
+  yyss = yyssa;
+  yyvs = yyvsa;
+  yystacksize = YYINITDEPTH;
+
   YYDPRINTF ((stderr, "Starting parse\n"));
 
   yystate = 0;
   yyerrstatus = 0;
   yynerrs = 0;
-  yychar = YYEMPTY;		/* Cause a token to be read.  */
+  yychar = YYEMPTY; /* Cause a token to be read.  */
 
   /* Initialize stack pointers.
      Waste one element of value and location stack
      so that they stay on the same level as the state stack.
      The wasted elements are never initialized.  */
-
   yyssp = yyss;
   yyvsp = yyvs;
 
@@ -1349,7 +1373,6 @@ yyparse ()
 	YYSTYPE *yyvs1 = yyvs;
 	yytype_int16 *yyss1 = yyss;
 
-
 	/* Each stack pointer address is followed by the size of the
 	   data in use in that stack, in bytes.  This used to be a
 	   conditional around just the two extra args, but that might
@@ -1357,7 +1380,6 @@ yyparse ()
 	yyoverflow (YY_("memory exhausted"),
 		    &yyss1, yysize * sizeof (*yyssp),
 		    &yyvs1, yysize * sizeof (*yyvsp),
-
 		    &yystacksize);
 
 	yyss = yyss1;
@@ -1380,9 +1402,8 @@ yyparse ()
 	  (union yyalloc *) YYSTACK_ALLOC (YYSTACK_BYTES (yystacksize));
 	if (! yyptr)
 	  goto yyexhaustedlab;
-	YYSTACK_RELOCATE (yyss);
-	YYSTACK_RELOCATE (yyvs);
-
+	YYSTACK_RELOCATE (yyss_alloc, yyss);
+	YYSTACK_RELOCATE (yyvs_alloc, yyvs);
 #  undef YYSTACK_RELOCATE
 	if (yyss1 != yyssa)
 	  YYSTACK_FREE (yyss1);
@@ -1393,7 +1414,6 @@ yyparse ()
       yyssp = yyss + yysize - 1;
       yyvsp = yyvs + yysize - 1;
 
-
       YYDPRINTF ((stderr, "Stack size increased to %lu\n",
 		  (unsigned long int) yystacksize));
 
@@ -1403,6 +1423,9 @@ yyparse ()
 
   YYDPRINTF ((stderr, "Entering state %d\n", yystate));
 
+  if (yystate == YYFINAL)
+    YYACCEPT;
+
   goto yybackup;
 
 /*-----------.
@@ -1411,16 +1434,16 @@ yyparse ()
 yybackup:
 
   /* Do appropriate processing given the current state.  Read a
-     look-ahead token if we need one and don't already have one.  */
+     lookahead token if we need one and don't already have one.  */
 
-  /* First try to decide what to do without reference to look-ahead token.  */
+  /* First try to decide what to do without reference to lookahead token.  */
   yyn = yypact[yystate];
   if (yyn == YYPACT_NINF)
     goto yydefault;
 
-  /* Not known => get a look-ahead token if don't already have one.  */
+  /* Not known => get a lookahead token if don't already have one.  */
 
-  /* YYCHAR is either YYEMPTY or YYEOF or a valid look-ahead symbol.  */
+  /* YYCHAR is either YYEMPTY or YYEOF or a valid lookahead symbol.  */
   if (yychar == YYEMPTY)
     {
       YYDPRINTF ((stderr, "Reading a token: "));
@@ -1452,20 +1475,16 @@ yybackup:
       goto yyreduce;
     }
 
-  if (yyn == YYFINAL)
-    YYACCEPT;
-
   /* Count tokens shifted since error; after three, turn off error
      status.  */
   if (yyerrstatus)
     yyerrstatus--;
 
-  /* Shift the look-ahead token.  */
+  /* Shift the lookahead token.  */
   YY_SYMBOL_PRINT ("Shifting", yytoken, &yylval, &yylloc);
 
-  /* Discard the shifted token unless it is eof.  */
-  if (yychar != YYEOF)
-    yychar = YYEMPTY;
+  /* Discard the shifted token.  */
+  yychar = YYEMPTY;
 
   yystate = yyn;
   *++yyvsp = yylval;
@@ -1505,7 +1524,9 @@ yyreduce:
   switch (yyn)
     {
         case 3:
-#line 70 "imap_csim.y"
+
+/* Line 1464 of yacc.c  */
+#line 69 "imap_csim.y"
     {
 		   g_strreplace(&_map_info->image_name, (yyvsp[(5) - (7)].id));
 		   g_free ((yyvsp[(5) - (7)].id));
@@ -1513,43 +1534,57 @@ yyreduce:
     break;
 
   case 8:
-#line 82 "imap_csim.y"
+
+/* Line 1464 of yacc.c  */
+#line 81 "imap_csim.y"
     {}
     break;
 
   case 9:
-#line 83 "imap_csim.y"
+
+/* Line 1464 of yacc.c  */
+#line 82 "imap_csim.y"
     { g_free ((yyvsp[(3) - (3)].id)); }
     break;
 
   case 10:
-#line 84 "imap_csim.y"
+
+/* Line 1464 of yacc.c  */
+#line 83 "imap_csim.y"
     { g_free ((yyvsp[(3) - (3)].id)); }
     break;
 
   case 11:
-#line 88 "imap_csim.y"
+
+/* Line 1464 of yacc.c  */
+#line 87 "imap_csim.y"
     {
 		   _map_info->old_image_width = (yyvsp[(3) - (3)].val);
 		}
     break;
 
   case 12:
-#line 94 "imap_csim.y"
+
+/* Line 1464 of yacc.c  */
+#line 93 "imap_csim.y"
     {
 		   _map_info->old_image_height = (yyvsp[(3) - (3)].val);
 		}
     break;
 
   case 13:
-#line 100 "imap_csim.y"
+
+/* Line 1464 of yacc.c  */
+#line 99 "imap_csim.y"
     {
 		  (yyval.val) = (gint) (yyvsp[(1) - (1)].value);
 		}
     break;
 
   case 14:
-#line 104 "imap_csim.y"
+
+/* Line 1464 of yacc.c  */
+#line 103 "imap_csim.y"
     {
 		  (yyval.val) = (gint) g_ascii_strtod ((yyvsp[(1) - (1)].id), NULL);
 		  g_free ((yyvsp[(1) - (1)].id));
@@ -1557,7 +1592,9 @@ yyreduce:
     break;
 
   case 15:
-#line 111 "imap_csim.y"
+
+/* Line 1464 of yacc.c  */
+#line 110 "imap_csim.y"
     {
 		   g_strreplace(&_map_info->title, (yyvsp[(5) - (6)].id));
 		   g_free ((yyvsp[(5) - (6)].id));
@@ -1565,14 +1602,18 @@ yyreduce:
     break;
 
   case 21:
-#line 127 "imap_csim.y"
+
+/* Line 1464 of yacc.c  */
+#line 126 "imap_csim.y"
     {
 		  g_free ((yyvsp[(2) - (3)].id));
 		}
     break;
 
   case 22:
-#line 133 "imap_csim.y"
+
+/* Line 1464 of yacc.c  */
+#line 132 "imap_csim.y"
     {
 		   g_strreplace(&_map_info->author, (yyvsp[(2) - (3)].id));
 		   g_free ((yyvsp[(2) - (3)].id));
@@ -1580,7 +1621,9 @@ yyreduce:
     break;
 
   case 23:
-#line 140 "imap_csim.y"
+
+/* Line 1464 of yacc.c  */
+#line 139 "imap_csim.y"
     {
 		   gchar *description;
 
@@ -1592,7 +1635,9 @@ yyreduce:
     break;
 
   case 26:
-#line 155 "imap_csim.y"
+
+/* Line 1464 of yacc.c  */
+#line 154 "imap_csim.y"
     {
 		   if (current_type != UNDEFINED)
 		      add_shape(current_object);
@@ -1600,7 +1645,9 @@ yyreduce:
     break;
 
   case 41:
-#line 182 "imap_csim.y"
+
+/* Line 1464 of yacc.c  */
+#line 181 "imap_csim.y"
     {
 		   if (!g_ascii_strcasecmp((yyvsp[(3) - (3)].id), "RECT")) {
 		      current_object = create_rectangle(0, 0, 0, 0);
@@ -1619,7 +1666,9 @@ yyreduce:
     break;
 
   case 42:
-#line 200 "imap_csim.y"
+
+/* Line 1464 of yacc.c  */
+#line 199 "imap_csim.y"
     {
 		   char *p;
 		   if (current_type == RECTANGLE) {
@@ -1680,7 +1729,9 @@ yyreduce:
     break;
 
   case 43:
-#line 260 "imap_csim.y"
+
+/* Line 1464 of yacc.c  */
+#line 259 "imap_csim.y"
     {
 		   if (current_type == UNDEFINED) {
 		      g_strreplace(&_map_info->default_url, (yyvsp[(3) - (3)].id));
@@ -1692,20 +1743,26 @@ yyreduce:
     break;
 
   case 44:
-#line 271 "imap_csim.y"
+
+/* Line 1464 of yacc.c  */
+#line 270 "imap_csim.y"
     {
 		}
     break;
 
   case 46:
-#line 277 "imap_csim.y"
+
+/* Line 1464 of yacc.c  */
+#line 276 "imap_csim.y"
     {
 		   g_free ((yyvsp[(2) - (2)].id));
 		}
     break;
 
   case 47:
-#line 283 "imap_csim.y"
+
+/* Line 1464 of yacc.c  */
+#line 282 "imap_csim.y"
     {
 		   object_set_comment(current_object, (yyvsp[(3) - (3)].id));
 		   g_free ((yyvsp[(3) - (3)].id));
@@ -1713,7 +1770,9 @@ yyreduce:
     break;
 
   case 48:
-#line 290 "imap_csim.y"
+
+/* Line 1464 of yacc.c  */
+#line 289 "imap_csim.y"
     {
 		   object_set_target(current_object, (yyvsp[(3) - (3)].id));
 		   g_free ((yyvsp[(3) - (3)].id));
@@ -1721,7 +1780,9 @@ yyreduce:
     break;
 
   case 49:
-#line 297 "imap_csim.y"
+
+/* Line 1464 of yacc.c  */
+#line 296 "imap_csim.y"
     {
 		   object_set_mouse_over(current_object, (yyvsp[(3) - (3)].id));
 		   g_free ((yyvsp[(3) - (3)].id));
@@ -1729,7 +1790,9 @@ yyreduce:
     break;
 
   case 50:
-#line 304 "imap_csim.y"
+
+/* Line 1464 of yacc.c  */
+#line 303 "imap_csim.y"
     {
 		   object_set_mouse_out(current_object, (yyvsp[(3) - (3)].id));
 		   g_free ((yyvsp[(3) - (3)].id));
@@ -1737,7 +1800,9 @@ yyreduce:
     break;
 
   case 51:
-#line 311 "imap_csim.y"
+
+/* Line 1464 of yacc.c  */
+#line 310 "imap_csim.y"
     {
 		   object_set_focus(current_object, (yyvsp[(3) - (3)].id));
 		   g_free ((yyvsp[(3) - (3)].id));
@@ -1745,7 +1810,9 @@ yyreduce:
     break;
 
   case 52:
-#line 318 "imap_csim.y"
+
+/* Line 1464 of yacc.c  */
+#line 317 "imap_csim.y"
     {
 		   object_set_blur(current_object, (yyvsp[(3) - (3)].id));
 		   g_free ((yyvsp[(3) - (3)].id));
@@ -1753,8 +1820,9 @@ yyreduce:
     break;
 
 
-/* Line 1267 of yacc.c.  */
-#line 1758 "y.tab.c"
+
+/* Line 1464 of yacc.c  */
+#line 1826 "y.tab.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -1765,7 +1833,6 @@ yyreduce:
 
   *++yyvsp = yyval;
 
-
   /* Now `shift' the result of the reduction.  Determine what state
      that goes to, based on the state we popped back to and the rule
      number reduced by.  */
@@ -1830,7 +1897,7 @@ yyerrlab:
 
   if (yyerrstatus == 3)
     {
-      /* If just tried and failed to reuse look-ahead token after an
+      /* If just tried and failed to reuse lookahead token after an
 	 error, discard it.  */
 
       if (yychar <= YYEOF)
@@ -1847,7 +1914,7 @@ yyerrlab:
 	}
     }
 
-  /* Else will try to reuse look-ahead token after shifting the error
+  /* Else will try to reuse lookahead token after shifting the error
      token.  */
   goto yyerrlab1;
 
@@ -1904,9 +1971,6 @@ yyerrlab1:
       YY_STACK_PRINT (yyss, yyssp);
     }
 
-  if (yyn == YYFINAL)
-    YYACCEPT;
-
   *++yyvsp = yylval;
 
 
@@ -1931,7 +1995,7 @@ yyabortlab:
   yyresult = 1;
   goto yyreturn;
 
-#ifndef yyoverflow
+#if !defined(yyoverflow) || YYERROR_VERBOSE
 /*-------------------------------------------------.
 | yyexhaustedlab -- memory exhaustion comes here.  |
 `-------------------------------------------------*/
@@ -1942,7 +2006,7 @@ yyexhaustedlab:
 #endif
 
 yyreturn:
-  if (yychar != YYEOF && yychar != YYEMPTY)
+  if (yychar != YYEMPTY)
      yydestruct ("Cleanup: discarding lookahead",
 		 yytoken, &yylval);
   /* Do not reclaim the symbols of the rule which action triggered
@@ -1968,7 +2032,9 @@ yyreturn:
 }
 
 
-#line 327 "imap_csim.y"
+
+/* Line 1684 of yacc.c  */
+#line 326 "imap_csim.y"
 
 
 static void
